@@ -7,20 +7,57 @@
 #       authors:
 #         Jerome Carnis, carnis_jerome@yahoo.fr
 
-"""Class definition which handles the detector config used for data acquisition."""
+"""
+Detector classes.
+
+These classes handles the detector config used for data acquisition. The available
+detectors are:
+
+- Maxipix
+- Eiger2M
+- Eiger4M
+- Timepix
+- Merlin
+- Dummy
+
+"""
+
+from abc import ABC, abstractmethod
 import numpy as np
-from numbers import Number, Real
+from numbers import Real
 import os
 
 from bcdi.utils import validation as valid
 
 
-class Detector:
+def create_detector(name, **kwargs):
+    """
+    Create a Detector instance depending on the detector.
+
+    :param name: str, name of the detector
+    :return:  the corresponding diffractometer instance
+    """
+    if name == "Maxipix":
+        return Maxipix(name=name, **kwargs)
+    if name == "Eiger2M":
+        return Eiger2M(name=name, **kwargs)
+    if name == "Eiger4M":
+        return Eiger4M(name=name, **kwargs)
+    if name == "Timepix":
+        return Timepix(name=name, **kwargs)
+    if name == "Merlin":
+        return Merlin(name=name, **kwargs)
+    if name == "Dummy":
+        return Dummy(name=name, **kwargs)
+    raise NotImplementedError(f"No implementation for the {name} detector")
+
+
+class Detector(ABC):
     """
     Class to handle the configuration of the detector used for data acquisition.
 
     :param name: name of the detector in {'Maxipix', 'Timepix', 'Merlin', 'Eiger2M',
-     'Eiger4M'}
+     'Eiger4M', 'Dummy'}
     :param datadir: directory where the data files are located
     :param savedir: directory where to save the results
     :param template_imagefile: beamline-dependent template for the data files
@@ -42,10 +79,6 @@ class Detector:
      (stacking dimension, detector vertical axis, detector horizontal axis)
     :param kwargs:
 
-     - 'is_series': boolean, True is the measurement is a series at PETRAIII P10
-       beamline
-     - 'nb_pixel_x' and 'nb_pixel_y': useful when part of the detector is broken
-       (less pixels than expected)
      - 'preprocessing_binning': tuple of the three binning factors used in a previous
        preprocessing step
      - 'offsets': tuple or list, sample and detector offsets corresponding to the
@@ -72,28 +105,10 @@ class Detector:
     ):
         # the detector name should be initialized first,
         # other properties are depending on it
-        self.name = name
-
-        valid.valid_kwargs(
-            kwargs=kwargs,
-            allowed_kwargs={
-                "is_series",
-                "nb_pixel_x",
-                "nb_pixel_y",
-                "pixel_size",
-                "preprocessing_binning",
-                "offsets",
-                "linearity_func",
-            },
-            name="Detector.__init__",
-        )
+        self._name = name
 
         # load the kwargs
-        self.is_series = kwargs.get("is_series", False)
         self.preprocessing_binning = kwargs.get("preprocessing_binning") or (1, 1, 1)
-        self.nb_pixel_x = kwargs.get("nb_pixel_x")
-        self.nb_pixel_y = kwargs.get("nb_pixel_y")
-        self.custom_pixelsize = kwargs.get("pixel_size")
         self.offsets = kwargs.get("offsets")  # delegate the test to xrayutilities
         linearity_func = kwargs.get("linearity_func")
         if linearity_func is not None and not callable(linearity_func):
@@ -115,6 +130,13 @@ class Detector:
         self.template_imagefile = template_imagefile
         self.specfile = specfile
 
+        # dictionary of keys: beamline_name and values: counter name for the image
+        # number in the log file.
+        self._counter_table = {}
+
+        # initialize the threshold for saturation, can be overriden in child classes
+        self.saturation_threshold = None
+
     @property
     def binning(self):
         """
@@ -139,18 +161,15 @@ class Detector:
         )
         self._binning = value
 
-    @property
-    def counter(self):
-        """Name of the counter for the image number."""
-        counter_dict = {
-            "Maxipix": "mpx4inr",
-            "Eiger2M": "ei2minr",
-            "Eiger4M": None,
-            "Timepix": None,
-            "Merlin": "alba2",
-            "Dummy": None,
-        }
-        return counter_dict.get(self.name)
+    def counter(self, beamline):
+        """
+        Name of the counter in the log file for the image number.
+
+        :param beamline: str, name of the beamline
+        """
+        if not isinstance(beamline, str):
+            raise TypeError("beamline should be a string")
+        return self._counter_table.get(beamline)
 
     @property
     def datadir(self):
@@ -168,66 +187,30 @@ class Detector:
         )
         self._datadir = value
 
-    @property
-    def is_series(self):
-        """Boolean, True for a series measurement at PETRAIII P10."""
-        return self._is_series
-
-    @is_series.setter
-    def is_series(self, value):
-        if not isinstance(value, bool):
-            raise TypeError("is_series should be a boolean")
-        self._is_series = value
-
     @property
     def name(self):
         """Name of the detector."""
         return self._name
 
-    @name.setter
-    def name(self, value):
-        valid_names = {"Maxipix", "Timepix", "Merlin", "Eiger2M", "Eiger4M", "Dummy"}
-        if value not in valid_names:
-            raise ValueError(f"Name should be in {valid_names}")
-        self._name = value
-
     @property
     def nb_pixel_x(self):
         """
         Horizontal number of pixels of the detector.
 
-        It takes into account an eventual preprocessing binning.
+        It takes into account an eventual preprocessing binning (useful when
+        reloading a already preprocessed file).
         """
-        return self._nb_pixel_x
-
-    @nb_pixel_x.setter
-    def nb_pixel_x(self, value):
-        if value is None:
-            value = self.pix_number[1]
-        if not isinstance(value, int):
-            raise TypeError("nb_pixel_x should be a positive integer")
-        if value <= 0:
-            raise ValueError("nb_pixel_x should be a positive integer")
-        self._nb_pixel_x = value // self.preprocessing_binning[2]
+        return self.unbinned_pixel_number[1] // self.preprocessing_binning[2]
 
     @property
     def nb_pixel_y(self):
         """
         Vertical number of pixels of the detector.
 
-        It takes into account an eventual preprocessing binning.
+        It takes into account an eventual preprocessing binning (useful when
+        reloading a already preprocessed file).
         """
-        return self._nb_pixel_y
-
-    @nb_pixel_y.setter
-    def nb_pixel_y(self, value):
-        if value is None:
-            value = self.pix_number[0]
-        if not isinstance(value, int):
-            raise TypeError("nb_pixel_y should be a positive integer")
-        if value <= 0:
-            raise ValueError("nb_pixel_y should be a positive integer")
-        self._nb_pixel_y = value // self.preprocessing_binning[1]
+        return self.unbinned_pixel_number[0] // self.preprocessing_binning[1]
 
     @property
     def params(self):
@@ -235,14 +218,13 @@ class Detector:
         return {
             "Class": self.__class__.__name__,
             "name": self.name,
-            "unbinned_pixel_m": self.unbinned_pixel,
+            "unbinned_pixel_size_m": self.unbinned_pixel_size,
             "nb_pixel_x": self.nb_pixel_x,
             "nb_pixel_y": self.nb_pixel_y,
             "binning": self.binning,
             "roi": self.roi,
             "sum_roi": self.sum_roi,
             "preprocessing_binning": self.preprocessing_binning,
-            "is_series": self.is_series,
             "rootdir": self.rootdir,
             "datadir": self.datadir,
             "scandir": self.scandir,
@@ -256,33 +238,20 @@ class Detector:
     @property
     def pixelsize_x(self):
         """Horizontal pixel size of the detector after taking into account binning."""
-        return self.unbinned_pixel[1] * self.preprocessing_binning[2] * self.binning[2]
+        return (
+            self.unbinned_pixel_size[1]
+            * self.preprocessing_binning[2]
+            * self.binning[2]
+        )
 
     @property
     def pixelsize_y(self):
         """Vertical pixel size of the detector after taking into account binning."""
-        return self.unbinned_pixel[0] * self.preprocessing_binning[1] * self.binning[1]
-
-    @property
-    def pix_number(self):
-        """
-        Define the number of pixels of the unbinned detector.
-
-        Convention: (vertical, horizontal)
-        """
-        if self.name in {"Maxipix", "Dummy"}:
-            number = (516, 516)
-        elif self.name == "Timepix":
-            number = (256, 256)
-        elif self.name == "Merlin":
-            number = (515, 515)
-        elif self.name == "Eiger2M":
-            number = (2164, 1030)
-        elif self.name == "Eiger4M":
-            number = (2167, 2070)
-        else:
-            number = None
-        return number
+        return (
+            self.unbinned_pixel_size[0]
+            * self.preprocessing_binning[1]
+            * self.binning[1]
+        )
 
     @property
     def preprocessing_binning(self):
@@ -442,40 +411,30 @@ class Detector:
         self._template_imagefile = value
 
     @property
-    def unbinned_pixel(self):
+    @abstractmethod
+    def unbinned_pixel_number(self):
+        """
+        Define the number of pixels of the unbinned detector.
+
+        Convention: (vertical, horizontal)
+        """
+
+    @property
+    @abstractmethod
+    def unbinned_pixel_size(self):
         """Pixel size (vertical, horizontal) of the unbinned detector in meters."""
-        if self.name in {"Maxipix", "Timepix", "Merlin"}:
-            pix = (55e-06, 55e-06)
-        elif self.name in {"Eiger2M", "Eiger4M"}:
-            pix = (75e-06, 75e-06)
-        elif self.name == "Dummy":
-            if self.custom_pixelsize is not None:
-                valid.valid_item(
-                    self.custom_pixelsize,
-                    allowed_types=Real,
-                    min_excluded=0,
-                    name="custom_pixelsize",
-                )
-                pix = (self.custom_pixelsize, self.custom_pixelsize)
-            else:
-                pix = (55e-06, 55e-06)
-                print(f"Defaulting the pixel size to {pix}")
-        else:
-            pix = None
-        return pix
 
     def __repr__(self):
         """Representation string of the Detector instance."""
         return (
             f"{self.__class__.__name__}(name='{self.name}', "
-            f"unbinned_pixel={self.unbinned_pixel}, "
+            f"unbinned_pixel_size={self.unbinned_pixel_size}, "
             f"nb_pixel_x={self.nb_pixel_x}, "
             f"nb_pixel_y={self.nb_pixel_y}, "
             f"binning={self.binning},\n"
             f"roi={self.roi}, "
             f"sum_roi={self.sum_roi}, "
             f"preprocessing_binning={self.preprocessing_binning}, "
-            f"is_series={self.is_series},\n"
             f"rootdir = {self.rootdir},\n"
             f"datadir = {self.datadir},\n"
             f"scandir = {self.scandir},\n"
@@ -486,8 +445,74 @@ class Detector:
             f" specfile = {self.specfile},\n"
         )
 
+    @staticmethod
+    def _background_subtraction(data, background):
+        """
+        Apply background subtraction to the data.
+
+        :param data: a 2D numpy ndarray
+        :param background: None or a 2D numpy array
+        :return: the corrected data array
+        """
+        if data.ndim != 2:
+            raise ValueError("data should be a 2D array")
+        if background is not None:
+            valid.valid_ndarray((data, background), ndim=2)
+            return data - background
+        return data
+
+    @staticmethod
+    def _flatfield_correction(data, flatfield):
+        """
+        Apply flatfield correction to the data.
+
+        :param data: a 2D numpy ndarray
+        :param flatfield: None or a 2D numpy array
+        :return: the corrected data array
+        """
+        if flatfield is not None:
+            valid.valid_ndarray((data, flatfield), ndim=2)
+            return np.multiply(flatfield, data)
+        return data
+
+    @staticmethod
+    def _hotpixels_correction(data, mask, hotpixels):
+        """
+        Apply hotpixels correction to the data and update the mask.
+
+        :param data: a 2D numpy ndarray
+        :param hotpixels: None or a 2D numpy array, 1 if the pixel needs to be masked,
+         0 otherwise
+        :return: the corrected data array
+        """
+        if hotpixels is not None:
+            valid.valid_ndarray((data, mask, hotpixels), ndim=2)
+            if ((hotpixels == 0).sum() + (hotpixels == 1).sum()) != hotpixels.size:
+                raise ValueError("hotpixels should be an array of 0 and 1")
+
+            data[hotpixels == 1] = 0
+            mask[hotpixels == 1] = 1
+
+        return data, mask
+
+    def _linearity_correction(self, data):
+        """
+        Apply a correction to data if the detector response is not linear.
+
+        :param data: a 2D numpy array
+        :return: the corrected data array
+        """
+        if self._linearity_func is not None:
+            if not callable(self._linearity_func):
+                raise TypeError("linearity_function is not callable")
+            valid.valid_ndarray(data, ndim=2)
+            data = data.astype(float)
+            nby, nbx = data.shape
+            return self._linearity_func(data.flatten()).reshape((nby, nbx))
+        return data
+
     def mask_detector(
-        self, data, mask, nb_img=1, flatfield=None, background=None, hotpixels=None
+        self, data, mask, nb_frames=1, flatfield=None, background=None, hotpixels=None
     ):
         """
         Mask data measured with a 2D detector.
@@ -497,148 +522,368 @@ class Detector:
 
         :param data: the 2D data to mask
         :param mask: the 2D mask to be updated
-        :param nb_img: number of images summed to yield the 2D data
-         (e.g. in a series measurement)
+        :param nb_frames: number of frames summed to yield the 2D data
+         (e.g. in a series measurement), used when defining the threshold for hot pixels
         :param flatfield: the 2D flatfield array to be multiplied with the data
         :param background: a 2D array to be subtracted to the data
         :param hotpixels: a 2D array with hotpixels to be masked
          (1=hotpixel, 0=normal pixel)
         :return: the masked data and the updated mask
         """
-        if not isinstance(data, np.ndarray) or not isinstance(mask, np.ndarray):
-            raise TypeError("data and mask should be numpy arrays")
-        if data.ndim != 2 or mask.ndim != 2:
-            raise ValueError("data and mask should be 2D arrays")
+        valid.valid_ndarray((data, mask), ndim=2)
 
-        if data.shape != mask.shape:
-            raise ValueError(
-                "data and mask must have the same shape\n data is ",
-                data.shape,
-                " while mask is ",
-                mask.shape,
-            )
-
-        # linearity correction
+        # linearity correctiondata
         if self._linearity_func is not None:
-            data = data.astype(float)
-            nby, nbx = data.shape
-            data = self._linearity_func(data.flatten()).reshape((nby, nbx))
+            data = self._linearity_func(data)
 
         # flatfield correction
-        if flatfield is not None:
-            if flatfield.shape != data.shape:
-                raise ValueError(
-                    "flatfield and data must have the same shape\n data is ",
-                    flatfield.shape,
-                    " while data is ",
-                    data.shape,
-                )
-            data = np.multiply(flatfield, data)
+        data = self._flatfield_correction(data=data, flatfield=flatfield)
 
         # remove the background
-        if background is not None:
-            if background.shape != data.shape:
-                raise ValueError(
-                    "background and data must have the same shape\n data is ",
-                    background.shape,
-                    " while data is ",
-                    data.shape,
-                )
-            data = data - background
+        data = self._background_subtraction(data=data, background=background)
 
         # mask hotpixels
-        if hotpixels is not None:
-            if hotpixels.shape != data.shape:
-                raise ValueError(
-                    "hotpixels and data must have the same shape\n data is ",
-                    hotpixels.shape,
-                    " while data is ",
-                    data.shape,
-                )
-            data[hotpixels == 1] = 0
-            mask[hotpixels == 1] = 1
+        data, mask = self._hotpixels_correction(
+            data=data, mask=mask, hotpixels=hotpixels
+        )
+        # mask detector gaps
+        data, mask = self._mask_gaps(data, mask)
 
-        if self.name == "Eiger2M":
-            data[:, 255:259] = 0
-            data[:, 513:517] = 0
-            data[:, 771:775] = 0
-            data[0:257, 72:80] = 0
-            data[255:259, :] = 0
-            data[511:552, :0] = 0
-            data[804:809, :] = 0
-            data[1061:1102, :] = 0
-            data[1355:1359, :] = 0
-            data[1611:1652, :] = 0
-            data[1905:1909, :] = 0
-            data[1248:1290, 478] = 0
-            data[1214:1298, 481] = 0
-            data[1649:1910, 620:628] = 0
-
-            mask[:, 255:259] = 1
-            mask[:, 513:517] = 1
-            mask[:, 771:775] = 1
-            mask[0:257, 72:80] = 1
-            mask[255:259, :] = 1
-            mask[511:552, :] = 1
-            mask[804:809, :] = 1
-            mask[1061:1102, :] = 1
-            mask[1355:1359, :] = 1
-            mask[1611:1652, :] = 1
-            mask[1905:1909, :] = 1
-            mask[1248:1290, 478] = 1
-            mask[1214:1298, 481] = 1
-            mask[1649:1910, 620:628] = 1
-
-            # mask hot pixels
-            mask[data > 1e6 * nb_img] = 1
-            data[data > 1e6 * nb_img] = 0
-
-        elif self.name == "Eiger4M":
-            data[:, 0:1] = 0
-            data[:, -1:] = 0
-            data[0:1, :] = 0
-            data[-1:, :] = 0
-            data[:, 1029:1041] = 0
-            data[513:552, :] = 0
-            data[1064:1103, :] = 0
-            data[1615:1654, :] = 0
-
-            mask[:, 0:1] = 1
-            mask[:, -1:] = 1
-            mask[0:1, :] = 1
-            mask[-1:, :] = 1
-            mask[:, 1029:1041] = 1
-            mask[513:552, :] = 1
-            mask[1064:1103, :] = 1
-            mask[1615:1654, :] = 1
-
-            # mask hot pixels, 4000000000 for the Eiger4M
-            mask[data > 4000000000 * nb_img] = 1
-            data[data > 4000000000 * nb_img] = 0
-
-        elif self.name == "Maxipix":
-            data[:, 255:261] = 0
-            data[255:261, :] = 0
-
-            mask[:, 255:261] = 1
-            mask[255:261, :] = 1
-
-            # mask hot pixels
-            mask[data > 1e6 * nb_img] = 1
-            data[data > 1e6 * nb_img] = 0
-
-        elif self.name == "Merlin":
-            data[:, 255:260] = 0
-            data[255:260, :] = 0
-
-            mask[:, 255:260] = 1
-            mask[255:260, :] = 1
-
-            # mask hot pixels
-            mask[data > 1e6 * nb_img] = 1
-            data[data > 1e6 * nb_img] = 0
-
-        elif self.name == "Timepix":
-            pass  # no gaps
+        # remove saturated pixels
+        data, mask = self._saturation_correction(data, mask, nb_frames=nb_frames)
 
         return data, mask
+
+    @staticmethod
+    def _mask_gaps(data, mask):
+        """
+        Mask the gaps between sensors in the detector.
+
+        :param data: a 2D numpy array
+        :param mask: a 2D numpy array of the same shape as data
+        :return:
+
+         - the masked data
+         - the updated mask
+
+        """
+        valid.valid_ndarray((data, mask), ndim=2)
+        return data, mask
+
+    def _saturation_correction(self, data, mask, nb_frames):
+        """
+        Mask pixels above a certain threshold.
+
+        This is detector dependent.
+
+        :param data: a 2D numpy array
+        :param mask: a 2D numpy array of the same shape as data
+        :param nb_frames: int, number of frames concatenated to obtain the 2D data array
+        :return:
+
+         - the masked data
+         - the updated mask
+
+        """
+        if self.saturation_threshold is not None:
+            valid.valid_ndarray((data, mask), ndim=2)
+
+            valid.valid_item(
+                nb_frames, allowed_types=int, min_excluded=0, name="nb_frames"
+            )
+            mask[data > self.saturation_threshold * nb_frames] = 1
+            data[data > self.saturation_threshold * nb_frames] = 0
+        return data, mask
+
+
+class Maxipix(Detector):
+    """Implementation of the Maxipix detector."""
+
+    def __init__(self, name, **kwargs):
+        super().__init__(name=name, **kwargs)
+        self._counter_table = {"ID01": "mpx4inr"}  # useful if the same type of detector
+        # is used at several beamlines
+        self.saturation_threshold = 1e6
+
+    @staticmethod
+    def _mask_gaps(data, mask):
+        """
+        Mask the gaps between sensors in the detector.
+
+        :param data: a 2D numpy array
+        :param mask: a 2D numpy array of the same shape as data
+        :return:
+
+         - the masked data
+         - the updated mask
+
+        """
+        valid.valid_ndarray((data, mask), ndim=2)
+
+        data[:, 255:261] = 0
+        data[255:261, :] = 0
+
+        mask[:, 255:261] = 1
+        mask[255:261, :] = 1
+        return data, mask
+
+    @property
+    def unbinned_pixel_number(self):
+        """
+        Define the number of pixels of the unbinned detector.
+
+        Convention: (vertical, horizontal)
+        """
+        return 516, 516
+
+    @property
+    def unbinned_pixel_size(self):
+        """Pixel size (vertical, horizontal) of the unbinned detector in meters."""
+        return 55e-06, 55e-06
+
+
+class Eiger2M(Detector):
+    """Implementation of the Eiger2M detector."""
+
+    def __init__(self, name, **kwargs):
+        super().__init__(name=name, **kwargs)
+        self._counter_table = {"ID01": "ei2minr"}  # useful if the same type of detector
+        # is used at several beamlines
+        self.saturation_threshold = 1e6
+
+    @staticmethod
+    def _mask_gaps(data, mask):
+        """
+        Mask the gaps between sensors in the detector.
+
+        :param data: a 2D numpy array
+        :param mask: a 2D numpy array of the same shape as data
+        :return:
+
+         - the masked data
+         - the updated mask
+
+        """
+        valid.valid_ndarray((data, mask), ndim=2)
+
+        data[:, 255:259] = 0
+        data[:, 513:517] = 0
+        data[:, 771:775] = 0
+        data[0:257, 72:80] = 0
+        data[255:259, :] = 0
+        data[511:552, :] = 0
+        data[804:809, :] = 0
+        data[1061:1102, :] = 0
+        data[1355:1359, :] = 0
+        data[1611:1652, :] = 0
+        data[1905:1909, :] = 0
+        data[1248:1290, 478] = 0
+        data[1214:1298, 481] = 0
+        data[1649:1910, 620:628] = 0
+
+        mask[:, 255:259] = 1
+        mask[:, 513:517] = 1
+        mask[:, 771:775] = 1
+        mask[0:257, 72:80] = 1
+        mask[255:259, :] = 1
+        mask[511:552, :] = 1
+        mask[804:809, :] = 1
+        mask[1061:1102, :] = 1
+        mask[1355:1359, :] = 1
+        mask[1611:1652, :] = 1
+        mask[1905:1909, :] = 1
+        mask[1248:1290, 478] = 1
+        mask[1214:1298, 481] = 1
+        mask[1649:1910, 620:628] = 1
+        return data, mask
+
+    @property
+    def unbinned_pixel_number(self):
+        """
+        Define the number of pixels of the unbinned detector.
+
+        Convention: (vertical, horizontal)
+        """
+        return 2164, 1030
+
+    @property
+    def unbinned_pixel_size(self):
+        """Pixel size (vertical, horizontal) of the unbinned detector in meters."""
+        return 75e-06, 75e-06
+
+
+class Eiger4M(Detector):
+    """Implementation of the Eiger4M detector."""
+
+    def __init__(self, name, **kwargs):
+        super().__init__(name=name, **kwargs)
+        self.saturation_threshold = 4000000000
+
+    @staticmethod
+    def _mask_gaps(data, mask):
+        """
+        Mask the gaps between sensors in the detector.
+
+        :param data: a 2D numpy array
+        :param mask: a 2D numpy array of the same shape as data
+        :return:
+
+         - the masked data
+         - the updated mask
+
+        """
+        valid.valid_ndarray((data, mask), ndim=2)
+
+        data[:, 0:1] = 0
+        data[:, -1:] = 0
+        data[0:1, :] = 0
+        data[-1:, :] = 0
+        data[:, 1029:1041] = 0
+        data[513:552, :] = 0
+        data[1064:1103, :] = 0
+        data[1615:1654, :] = 0
+
+        mask[:, 0:1] = 1
+        mask[:, -1:] = 1
+        mask[0:1, :] = 1
+        mask[-1:, :] = 1
+        mask[:, 1029:1041] = 1
+        mask[513:552, :] = 1
+        mask[1064:1103, :] = 1
+        mask[1615:1654, :] = 1
+        return data, mask
+
+    @property
+    def unbinned_pixel_number(self):
+        """
+        Define the number of pixels of the unbinned detector.
+
+        Convention: (vertical, horizontal)
+        """
+        return 2167, 2070
+
+    @property
+    def unbinned_pixel_size(self):
+        """Pixel size (vertical, horizontal) of the unbinned detector in meters."""
+        return 75e-06, 75e-06
+
+
+class Timepix(Detector):
+    """Implementation of the Timepix detector."""
+
+    def __init__(self, name, **kwargs):
+        super().__init__(name=name, **kwargs)
+
+    @property
+    def unbinned_pixel_number(self):
+        """
+        Define the number of pixels of the unbinned detector.
+
+        Convention: (vertical, horizontal)
+        """
+        return 256, 256
+
+    @property
+    def unbinned_pixel_size(self):
+        """Pixel size (vertical, horizontal) of the unbinned detector in meters."""
+        return 55e-06, 55e-06
+
+
+class Merlin(Detector):
+    """Implementation of the Merlin detector."""
+
+    def __init__(self, name, **kwargs):
+        super().__init__(name=name, **kwargs)
+        self.saturation_threshold = 1e6
+
+    @staticmethod
+    def _mask_gaps(data, mask):
+        """
+        Mask the gaps between sensors in the detector.
+
+        :param data: a 2D numpy array
+        :param mask: a 2D numpy array of the same shape as data
+        :return:
+
+         - the masked data
+         - the updated mask
+
+        """
+        valid.valid_ndarray((data, mask), ndim=2)
+
+        data[:, 255:260] = 0
+        data[255:260, :] = 0
+
+        mask[:, 255:260] = 1
+        mask[255:260, :] = 1
+        return data, mask
+
+    @property
+    def unbinned_pixel_number(self):
+        """
+        Define the number of pixels of the unbinned detector.
+
+        Convention: (vertical, horizontal)
+        """
+        return 515, 515
+
+    @property
+    def unbinned_pixel_size(self):
+        """Pixel size (vertical, horizontal) of the unbinned detector in meters."""
+        return 55e-06, 55e-06
+
+
+class Dummy(Detector):
+    """
+    Implementation of the Dummy detector.
+
+    :param kwargs:
+     - 'custom_pixelnumber': (V, H) number of pixels of the unbinned dummy detector, as
+       a tuple of two positive integers.
+     - 'custom_pixelsize': float, pixel size of the dummy detector in m.
+
+    """
+
+    def __init__(self, name, **kwargs):
+
+        self.custom_pixelsize = kwargs.get("custom_pixelsize")
+        valid.valid_item(
+            self.custom_pixelsize,
+            allowed_types=Real,
+            min_excluded=0,
+            allow_none=True,
+            name="custom_pixelsize",
+        )
+        self.custom_pixelnumber = kwargs.get("custom_pixelnumber")
+        valid.valid_container(
+            self.custom_pixelnumber,
+            container_types=(list, tuple, np.ndarray),
+            length=2,
+            item_types=int,
+            min_excluded=0,
+            allow_none=True,
+            name="custom_pixelnumber",
+        )
+        super().__init__(name=name, **kwargs)
+
+    @property
+    def unbinned_pixel_number(self):
+        """
+        Define the number of pixels of the unbinned detector.
+
+        Convention: (vertical, horizontal)
+        """
+        if self.custom_pixelnumber is not None and all(
+            val is not None for val in self.custom_pixelnumber
+        ):
+            return self.custom_pixelnumber
+        print(f"Defaulting the pixel size to {516, 516}")
+        return 516, 516
+
+    @property
+    def unbinned_pixel_size(self):
+        """Pixel size (vertical, horizontal) of the unbinned detector in meters."""
+        if self.custom_pixelsize is not None:
+            return self.custom_pixelsize, self.custom_pixelsize
+        print(f"Defaulting the pixel size to {55e-06, 55e-06}")
+        return 55e-06, 55e-06
