@@ -8,7 +8,8 @@
 #         Jerome Carnis, carnis_jerome@yahoo.fr
 """Functions related to the validation of input parameters."""
 
-from numbers import Real
+from collections import Sequence
+from numbers import Real, Number
 import numpy as np
 
 
@@ -41,6 +42,7 @@ def valid_container(
     :param max_excluded: maximum allowed value (exclusive)
     :param allow_none: True if the container items are allowed to be None
     :param name: name of the calling object appearing in exception messages
+    :return: True if checks pass, raise some error otherwise
     """
     supported_containers = {list, tuple, set, str, np.ndarray, dict}
 
@@ -175,6 +177,7 @@ def valid_kwargs(kwargs, allowed_kwargs, name=None):
     :param kwargs: dictionnary of kwargs to check
     :param allowed_kwargs: set of allowed keys
     :param name: name of the calling object appearing in exception messages
+    :return: True if checks pass, raise some error otherwise
     """
     # check the validity of the parameters
     if not isinstance(kwargs, dict):
@@ -228,6 +231,7 @@ def valid_item(
     :param max_excluded: maximum allowed value (exclusive)
     :param allow_none: True if the container items are allowed to be None
     :param name: name of the calling object appearing in exception messages
+    :return: True if checks pass, raise some error otherwise
     """
     # check the validity of the requirements
     if allowed_types is None:
@@ -320,3 +324,71 @@ def valid_item(
 
     # every tests passed, return True
     return True
+
+
+def valid_ndarray(arrays, ndim=None, shape=None, fix_ndim=True, fix_shape=True):
+    """
+    Check that arrays have the same shape and the correct number of dimensions.
+
+    :param arrays: a sequence of numpy ndarrays
+    :param ndim: int, the number of dimensions to be compared with
+    :param shape: sequence of int, shape to be comared with
+    :param fix_ndim: bool, if True the shape of all arrays should be equal
+    :param fix_shape: bool, if True the shape of all arrays should be equal
+    :return: True if checks pass, raise some error otherwise
+    """
+    # check the validity of the requirements
+    if isinstance(ndim, Number):
+        ndim = (ndim,)
+    valid_container(
+        ndim,
+        container_types=(tuple, list),
+        item_types=int,
+        min_excluded=0,
+        allow_none=True,
+        name="ndim",
+    )
+    valid_container(
+        shape,
+        container_types=(tuple, list),
+        item_types=int,
+        min_excluded=0,
+        allow_none=True,
+        name="shape",
+    )
+    if isinstance(arrays, np.ndarray):
+        arrays = (arrays,)
+    valid_container(
+        arrays,
+        container_types=(tuple, list),
+        item_types=np.ndarray,
+        min_length=1,
+        name="arrays",
+    )
+    if not isinstance(fix_ndim, bool):
+        raise TypeError(f"fix_ndim should be a boolean, got {type(fix_shape)}")
+    if not isinstance(fix_shape, bool):
+        raise TypeError(f"fix_shape should be a boolean, got {type(fix_shape)}")
+
+    # check the number of dimensions
+    if ndim is None:
+        ndim = (arrays[0].ndim,)
+    if not all(array.ndim in ndim for array in arrays):
+        raise ValueError(f"all arrays should have a number of dimensions in {ndim}")
+    if fix_ndim:
+        if not all(array.ndim == arrays[0].ndim for array in arrays):
+            raise ValueError(
+                "all arrays should have the same number of dimensions"
+                f" {arrays[0].ndim}"
+            )
+    else:
+        fix_shape = False
+
+    # check the shapes
+    if shape is None or any(val is None for val in shape):
+        shape = arrays[0].shape
+    if fix_shape and not all(array.shape == shape for array in arrays):
+        raise ValueError(f"all arrays should have the same shape {shape}")
+
+    # every tests passed, return True
+    return True
