@@ -22,7 +22,7 @@ from tkinter import filedialog
 from numpy.fft import fftn, fftshift
 import gc
 import bcdi.graph.graph_utils as gu
-from bcdi.experiment.detector import Detector
+from bcdi.experiment.detector import create_detector
 from bcdi.experiment.setup import Setup
 import bcdi.facet_recognition.facet_utils as fu
 import bcdi.preprocessing.preprocessing_utils as pru
@@ -269,7 +269,7 @@ my_cmap = colormap.cmap
 #######################
 # Initialize detector #
 #######################
-detector = Detector(
+detector = create_detector(
     name=detector, datadir="", template_imagefile=template_imagefile, roi=roi_detector
 )
 
@@ -298,11 +298,10 @@ setup = Setup(
 #############################################
 # Initialize geometry for orthogonalization #
 #############################################
-qconv, offsets = pru.init_qconversion(setup)
+qconv, offsets = setup.init_qconversion()
 detector.offsets = offsets
-hxrd = xu.experiment.HXRD(
-    sample_inplane, sample_outofplane, qconv=qconv
-)  # x downstream, y outboard, z vertical
+hxrd = xu.experiment.HXRD(sample_inplane, sample_outofplane, qconv=qconv)
+# x downstream, y outboard, z vertical
 # first two arguments in HXRD are the inplane reference direction
 # along the beam and surface normal of the sample
 cch1 = cch1 - detector.roi[0]  # take into account the roi if the image is cropped
@@ -343,8 +342,8 @@ else:
 detector.savedir = homedir
 
 if not reconstructed_data:  # load reciprocal space data
-    flatfield = pru.load_flatfield(flatfield_file)
-    hotpix_array = pru.load_hotpixels(hotpixels_file)
+    flatfield = util.load_flatfield(flatfield_file)
+    hotpix_array = util.load_hotpixels(hotpixels_file)
     logfile = setup.create_logfile(
         scan_number=scan, root_folder=root_folder, filename=specfile_name
     )
