@@ -15,10 +15,11 @@ import numpy as np
 import matplotlib.pyplot as plt
 from scipy.interpolate import interp1d
 from scipy.ndimage.measurements import center_of_mass
-from bcdi.experiment.detector import Detector
+from bcdi.experiment.detector import create_detector
 from bcdi.experiment.setup import Setup
 import bcdi.preprocessing.preprocessing_utils as pru
 import bcdi.graph.graph_utils as gu
+import bcdi.utils.utilities as util
 import bcdi.utils.validation as valid
 
 helptext = """
@@ -112,10 +113,6 @@ template_imagefile = "_master.h5"
 # template for P10: '_master.h5'
 # template for NANOMAX: '%06d.h5'
 # template for 34ID: 'Sample%dC_ES_data_51_256_256.npz'
-nb_pixel_x = None  # fix to declare a known detector but with less pixels
-# (e.g. one tile HS), leave None otherwise
-nb_pixel_y = None  # fix to declare a known detector but with less pixels
-# (e.g. one tile HS), leave None otherwise
 ####################################
 # q calculation related parameters #
 ####################################
@@ -179,16 +176,10 @@ check_roi = []  # a small ROI around the Bragg peak will be stored for each scan
 #######################
 # Initialize detector #
 #######################
-kwargs = {
-    "is_series": is_series,
-    "nb_pixel_x": nb_pixel_x,  # fix to declare a known detector but with less pixels
-    # (e.g. one tile HS)
-    "nb_pixel_y": nb_pixel_y,  # fix to declare a known detector but with less pixels
-    # (e.g. one tile HS)
-}
-
-detector = Detector(
-    name=detector, template_imagefile=template_imagefile, roi=roi_detector, **kwargs
+detector = create_detector(
+    name=detector,
+    template_imagefile=template_imagefile,
+    roi=roi_detector,
 )
 
 ####################
@@ -205,6 +196,7 @@ setup = Setup(
     custom_images=custom_images,
     custom_monitor=custom_monitor,
     custom_motors=custom_motors,
+    is_series=is_series,
 )
 
 ########################################
@@ -218,8 +210,8 @@ print(detector)
 ###############################################
 # load recursively the scans and update lists #
 ###############################################
-flatfield = pru.load_flatfield(flatfield_file)
-hotpix_array = pru.load_hotpixels(hotpixels_file)
+flatfield = util.load_flatfield(flatfield_file)
+hotpix_array = util.load_hotpixels(hotpixels_file)
 
 for scan_idx, scan_nb in enumerate(scans, start=1):
 
