@@ -27,7 +27,7 @@ import tkinter as tk
 from tkinter import filedialog
 import gc
 import bcdi.graph.graph_utils as gu
-from bcdi.experiment.detector import Detector
+from bcdi.experiment.detector import create_detector
 from bcdi.experiment.setup import Setup
 import bcdi.postprocessing.postprocessing_utils as pu
 import bcdi.preprocessing.preprocessing_utils as pru
@@ -57,9 +57,10 @@ scans = 76  # np.arange(1401, 1419+1, 3)  # scan number or list of scan numbers
 # bad_indices = np.argwhere(scans == 738)
 # scans = np.delete(scans, bad_indices)
 
-root_folder = "C:/Users/Jerome/Documents/data/P10_Longfei_Nov2020/data/"
+root_folder = "C:/Users/carnisj/Documents/data/P10_Longfei_Nov2020/data/"
 # folder of the experiment, where all scans are stored
-save_dir = None  # images will be saved here, leave it to None otherwise
+save_dir = root_folder + "test/"  # images will be saved here, leave it to None
+# otherwise
 data_dirname = None  # leave None to use the beamline default,
 # '' empty string when there is no subfolder
 # (data directly in the scan folder), or a non-empty string for the subfolder name
@@ -70,7 +71,7 @@ sample_name = "B15_syn_S1_2"  # str or list of str of sample names
 # If only one name is indicated, it will be repeated to match the number of scans.
 user_comment = ""  # string, should start with "_"
 debug = False  # set to True to see plots
-binning = (1, 1, 1)  # binning to apply to the data
+binning = (1, 2, 2)  # binning to apply to the data
 # (stacking dimension, detector vertical axis, detector horizontal axis)
 ##############################
 # parameters used in masking #
@@ -116,14 +117,14 @@ medfilt_order = 7  # for custom median filter,
 #################################################
 # parameters used when reloading processed data #
 #################################################
-reload_previous = True  # True to resume a previous masking (load data and mask)
+reload_previous = False  # True to resume a previous masking (load data and mask)
 reload_orthogonal = (
     False  # True if the reloaded data is already intepolated in an orthonormal frame
 )
 preprocessing_binning = (
     1,
-    2,
-    2,
+    1,
+    1,
 )  # binning factors in each dimension of the binned data to be reloaded
 ##################
 # saving options #
@@ -177,15 +178,16 @@ linearity_func = (
 )
 # 5.067e-11*array_1d**2 - 6.022e-07*array_1d + 0.889)) # MIR
 # np.divide(array_1d, (1-array_1d*1.3e-6))  # Sarah_1
-# (array_1d*(7.484e-22*array_1d**4 - 3.447e-16*array_1d**3 + 5.067e-11*array_1d**2 - 6.022e-07*array_1d + 0.889)) # MIR
+# (array_1d*(7.484e-22*array_1d**4 - 3.447e-16*array_1d**3 +
+# 5.067e-11*array_1d**2 - 6.022e-07*array_1d + 0.889)) # MIR
 # linearity correction for the detector, leave None otherwise.
 # You can use def instead of a lambda expression but the input array should be 1d
 # (flattened 2D detector array).
-x_bragg = 414  # horizontal pixel number of the Bragg peak,
+x_bragg = 1577  # horizontal pixel number of the Bragg peak,
 # can be used for the definition of the ROI
-y_bragg = 842  # vertical pixel number of the Bragg peak,
+y_bragg = 833  # vertical pixel number of the Bragg peak,
 # can be used for the definition of the ROI
-roi_detector = None  # [y_bragg - 216, y_bragg + 216, x_bragg - 200, x_bragg + 200]  #
+roi_detector = [y_bragg - 216, y_bragg + 216, x_bragg - 200, x_bragg + 200]  #
 # [Vstart, Vstop, Hstart, Hstop]
 # leave None to use the full detector.
 # Use with center_fft='skip' if you want this exact size.
@@ -229,9 +231,10 @@ beam_direction = (
     0,
 )  # beam direction in the laboratory frame (downstream, vertical up, outboard)
 sample_offsets = (
-    90,
     0,
     0,
+    90,
+    0,
 )  # tuple of offsets in degrees of the sample for each sample circle (outer first).
 # convention: the sample offsets will be subtracted to the motor values.
 # Leave None if no offset.
@@ -338,7 +341,9 @@ def press_key(event):
 
     :param event: button press event
     """
-    global original_data, original_mask, updated_mask, data, mask, frame_index, width, flag_aliens, flag_mask, flag_pause, xy, fig_mask, max_colorbar, ax0, ax1, ax2, ax3, previous_axis, info_text, my_cmap
+    global original_data, original_mask, updated_mask, data, mask, frame_index, width
+    global flag_aliens, flag_mask, flag_pause, xy, fig_mask, max_colorbar, ax0, ax1
+    global ax2, ax3, previous_axis, info_text, my_cmap
 
     try:
         if event.inaxes == ax0:
@@ -543,22 +548,13 @@ plt.rcParams["keymap.fullscreen"] = [""]
 #######################
 # Initialize detector #
 #######################
-kwargs = {
-    "is_series": is_series,
-    "preprocessing_binning": preprocessing_binning,
-    "nb_pixel_x": nb_pixel_x,  # fix to declare a known detector but with less pixels
-    # (e.g. one tile HS)
-    "nb_pixel_y": nb_pixel_y,  # fix to declare a known detector but with less pixels
-    # (e.g. one tile HS)
-    "linearity_func": linearity_func,
-}
-
-detector = Detector(
+detector = create_detector(
     name=detector,
     template_imagefile=template_imagefile,
     roi=roi_detector,
     binning=binning,
-    **kwargs,
+    preprocessing_binning=preprocessing_binning,
+    linearity_func=linearity_func,
 )
 
 ####################
@@ -580,6 +576,7 @@ setup = Setup(
     custom_monitor=custom_monitor,
     custom_motors=custom_motors,
     actuators=actuators,
+    is_series=is_series,
 )
 
 ########################################
@@ -761,9 +758,9 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
 
     else:  # new masking process
         reload_orthogonal = False  # the data is in the detector plane
-        flatfield = pru.load_flatfield(flatfield_file)
-        hotpix_array = pru.load_hotpixels(hotpixels_file)
-        background = pru.load_background(background_file)
+        flatfield = util.load_flatfield(flatfield_file)
+        hotpix_array = util.load_hotpixels(hotpixels_file)
+        background = util.load_background(background_file)
 
         data, mask, frames_logical, monitor = pru.load_bcdi_data(
             logfile=logfile,
@@ -834,7 +831,7 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
             gc.collect()
 
             if interp_method == "xrayutilities":
-                qconv, offsets = pru.init_qconversion(setup)
+                qconv, offsets = setup.init_qconversion()
                 detector.offsets = offsets
                 hxrd = xu.experiment.HXRD(
                     sample_inplane, sample_outofplane, en=energy, qconv=qconv
@@ -865,8 +862,8 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
                 )
                 # detector init_area method, pixel sizes are the binned ones
                 hxrd.Ang2Q.init_area(
-                    setup.detector_ver,
-                    setup.detector_hor,
+                    setup.detector_ver_xrutil,
+                    setup.detector_hor_xrutil,
                     cch1=cch1,
                     cch2=cch2,
                     Nch1=nch1,
